@@ -4,9 +4,10 @@ import time
 from collections import namedtuple
 from datetime import timedelta
 from distutils.version import StrictVersion
-from os import get_terminal_size
+from os import chdir, get_terminal_size, path
+from pathlib import Path
 from queue import Empty
-from random import randint
+from random import randint, sample
 
 import requests
 import soco
@@ -14,7 +15,8 @@ import soco.alarms
 import tabulate
 import xmltodict
 
-from .play_local_file import play_local_file
+from .m3u_parser import parse_m3u
+from .play_local_file import is_supported_type, play_local_file
 from .speaker_info import print_speaker_table
 from .utils import (
     convert_to_seconds,
@@ -566,29 +568,16 @@ def play_favourite_radio(speaker, action, args, soco_function, use_local_speaker
 @one_or_two_parameters
 def play_uri(speaker, action, args, soco_function, use_local_speaker_list):
     uri = args[0]
-    try:
-        # First try the simple case
-        if len(args) == 1:
-            speaker.play_uri(uri)
-        else:
-            # Note that 'title' parameter is ignored in the testing
-            # I've done, but try it anyway
-            speaker.play_uri(uri, title=args[1])
-    except:
-        # Now try setting to Radio
+    title = "" if len(args) == 1 else args[1]
+    for radio in [False, True]:
         try:
-            if len(args) == 1:
-                speaker.play_uri(uri, force_radio=True)
-            else:
-                speaker.play_uri(
-                    uri,
-                    title=args[1],
-                    force_radio=True,
-                )
+            speaker.play_uri(uri, title=title, force_radio=radio)
+            return True
         except:
-            error_and_exit("Failed to play URI: '{}'".format(uri))
-            return False
-    return True
+            continue
+
+    error_and_exit("Failed to play URI: '{}'".format(uri))
+    return False
 
 
 @zero_or_one_parameter
@@ -1701,6 +1690,62 @@ def play_file(speaker, action, args, soco_function, use_local_speaker_list):
     return play_local_file(speaker, args[0])
 
 
+@one_or_two_parameters
+def play_m3u(speaker, action, args, soco_function, use_local_speaker_list):
+    m3u_file = args[0]
+    options = "" if len(args) == 1 else args[1]
+    options = options.lower()
+
+    if not m3u_file.lower().endswith(".m3u"):
+        error_and_exit(
+            "Filename '{}' does not end in '.m3u' or '.M3U'".format(m3u_file)
+        )
+        return False
+
+    if not path.exists(m3u_file):
+        error_and_exit("File '{}' not found".format(m3u_file))
+        return False
+
+    logging.info("Parsing M3U file '{}'".format(m3u_file))
+    tracks = parse_m3u(m3u_file)
+    if not tracks:
+        error_and_exit("No tracks found in '{}'".format(m3u_file))
+
+    logging.info("Found {} tracks to play".format(len(tracks)))
+
+    if "s" in options:
+        logging.info("Shuffling playlist")
+        # For some reason, 'shuffle(tracks)' does not work
+        tracks = sample(tracks, len(tracks))
+
+    directory, _ = path.split(m3u_file)
+    if directory != "":
+        chdir(directory)
+
+    pad = len(str(len(tracks)))
+    for index, track in enumerate(tracks):
+        abs_filename = str(Path(track.path).absolute())
+        logging.info("Convert '{}' to '{}'".format(track.path, abs_filename))
+
+        if not path.exists(abs_filename):
+            print("Error: file not found:", abs_filename)
+            continue
+
+        if not is_supported_type(abs_filename):
+            print("Error: unsupported file type:", abs_filename)
+            continue
+
+        if "p" in options:
+            print(
+                "Playing {} of {}:".format(str(index + 1).zfill(pad), len(tracks)),
+                abs_filename,
+            )
+
+        play_local_file(speaker, abs_filename)
+
+    return True
+
+
 def process_action(speaker, action, args, use_local_speaker_list):
     sonos_function = actions.get(action, None)
     if sonos_function:
@@ -1972,4 +2017,6 @@ actions = {
     "rename": SonosFunction(rename, ""),
     "play_file": SonosFunction(play_file, ""),
     "play_local_file": SonosFunction(play_file, ""),
+    "play_m3u": SonosFunction(play_m3u, ""),
+    "play_local_m3u": SonosFunction(play_m3u, ""),
 }
