@@ -1,30 +1,43 @@
 import functools
 import logging
+import urllib.parse
 from http.server import HTTPServer
-from RangeHTTPServer import RangeRequestHandler
 from ipaddress import IPv4Address, IPv4Network
 from os import path
 from queue import Empty
 from threading import Thread
+
 import ifaddr
-import urllib.parse
+from RangeHTTPServer import RangeRequestHandler
+
 from .utils import error_and_exit, set_speaker_playing_local_file
 
 # The port range to use
 PORT_START = 54000
 PORT_END = 54099
 
+SUPPORTED_TYPES = ["MP3", "M4A", "MP4", "FLAC", "OGG", "WAV"]
+
 
 class MyHTTPHandler(RangeRequestHandler):
     filename = None
+    speaker_ip = None
 
     def do_GET(self):
         logging.info("Get request received by HTTP server")
 
-        # Only serve the specific file requested on the command line
-        if MyHTTPHandler.filename != self.path.replace("/", ""):
+        # Only serve the specific file requested on the command line,
+        # and only to the specific Sonos speaker IP address
+        if (
+            MyHTTPHandler.filename != self.path.replace("/", "")
+            or self.client_address[0] != MyHTTPHandler.speaker_ip
+        ):
             RangeRequestHandler.send_error(self, code=403, message="Access forbidden")
-            logging.info("Access to '{}' forbidden".format(self.path))
+            logging.info(
+                "Access to '{}' from '{}' forbidden".format(
+                    self.path, self.client_address[0]
+                )
+            )
             return
 
         try:
@@ -38,12 +51,14 @@ class MyHTTPHandler(RangeRequestHandler):
         return
 
 
-def http_server(server_ip, directory, filename):
+def http_server(server_ip, directory, filename, speaker_ip):
     # Set the directory from which to serve files, in the handler
     handler = functools.partial(MyHTTPHandler, directory=directory)
 
-    # Set up the only filename that will be served
+    # Set up the only filename that will be served, and the only IP to which
+    # it will be served
     MyHTTPHandler.filename = filename
+    MyHTTPHandler.speaker_ip = speaker_ip
 
     # Set up MIME types
     # MyHTTPHandler.extensions_map[".m4a"] = "audio/x-m4a"
@@ -97,6 +112,16 @@ def wait_until_stopped(speaker):
             pass
 
 
+def is_supported_type(filename):
+    file_upper = filename.upper()
+    for type in SUPPORTED_TYPES:
+        if file_upper.endswith("." + type):
+            # Supported file type
+            return True
+    else:
+        return False
+
+
 def play_local_file(speaker, pathname):
     # speaker is a SoCo instance
     # pathname is the local file to be played
@@ -107,16 +132,9 @@ def play_local_file(speaker, pathname):
 
     directory, filename = path.split(pathname)
 
-    supported_types = ["MP3", "M4A", "MP4", "FLAC", "OGG", "WAV"]
-    file_upper = filename.upper()
-    for type in supported_types:
-        if file_upper.endswith("." + type):
-            # Supported file type
-            break
-    else:
-        # Unsupported file type
+    if not is_supported_type(filename):
         error_and_exit(
-            "Unsupported file type; must be one of: {}".format(supported_types)
+            "Unsupported file type; must be one of: {}".format(SUPPORTED_TYPES)
         )
         return False
 
@@ -130,11 +148,16 @@ def play_local_file(speaker, pathname):
     logging.info("Using server IP address: {}".format(server_ip))
 
     # Start the webserver (runs in a daemon thread)
-    httpd = http_server(server_ip, directory, url_filename)
+    httpd = http_server(server_ip, directory, url_filename, speaker.ip_address)
     if not httpd:
         error_and_exit("Cannot create HTTP server")
         return False
 
+    # This ensures that other running invocations of 'play_file'
+    # receive their stop events, and terminate.
+    logging.info("Stopping speaker '{}'".format(speaker.player_name))
+    speaker.stop()
+
     # Assemble the URI and send to the speaker for playback
     uri = "http://" + server_ip + ":" + str(httpd.server_port) + "/" + url_filename
     logging.info("Playing file '{}' from directory '{}'".format(filename, directory))
